@@ -2,7 +2,6 @@
 
 from __future__ import annotations
 
-import io
 import json
 import os
 import pathlib
@@ -10,7 +9,7 @@ import tempfile
 import warnings
 from importlib import import_module
 from typing import Iterator
-from unittest.mock import patch
+from unittest.mock import MagicMock, patch
 
 import pytest
 from PIL import Image
@@ -43,6 +42,7 @@ from unstructured.documents.elements import (
     Text,
     Title,
 )
+from unstructured.file_utils.filetype import detect_filetype
 from unstructured.file_utils.model import FileType
 from unstructured.partition.auto import _PartitionerLoader, partition
 from unstructured.partition.common import UnsupportedFileFormatError
@@ -176,7 +176,7 @@ def test_partition_forwards_strategy_arg_to_partition_docx_and_its_brokers(
 
 EXPECTED_EMAIL_OUTPUT = [
     NarrativeText(text="This is a test email to use for unit tests."),
-    Title(text="Important points:"),
+    Text(text="Important points:"),
     ListItem(text="Roses are red"),
     ListItem(text="Violets are blue"),
 ]
@@ -441,7 +441,7 @@ def test_partition_md_from_url_works_with_embedded_html():
 def test_auto_partition_msg_from_filename():
     assert partition(example_doc_path("fake-email.msg"), strategy=PartitionStrategy.HI_RES) == [
         NarrativeText(text="This is a test email to use for unit tests."),
-        Title(text="Important points:"),
+        Text(text="Important points:"),
         ListItem(text="Roses are red"),
         ListItem(text="Violets are blue"),
     ]
@@ -561,7 +561,6 @@ def test_auto_partition_pdf_with_fast_strategy(request: FixtureRequest):
         strategy=PartitionStrategy.FAST,
         languages=None,
         metadata_filename=None,
-        include_page_breaks=False,
         infer_table_structure=False,
         extract_images_in_pdf=False,
         extract_image_block_types=None,
@@ -572,6 +571,33 @@ def test_auto_partition_pdf_with_fast_strategy(request: FixtureRequest):
     )
 
 
+@pytest.mark.parametrize("infer_bool", [True, False])
+def test_auto_handles_kwarg_with_infer_table_structure(infer_bool):
+    with patch(
+        "unstructured.partition.pdf_image.ocr.process_file_with_ocr",
+    ) as mock_process_file_with_model:
+        partition(
+            example_doc_path("pdf/layout-parser-paper-fast.pdf"),
+            pdf_infer_table_structure=True,
+            strategy=PartitionStrategy.HI_RES,
+            infer_table_structure=infer_bool,
+        )
+        assert mock_process_file_with_model.call_args[1]["infer_table_structure"] is infer_bool
+
+
+def test_auto_handles_kwarg_with_infer_table_structure_when_none():
+    with patch(
+        "unstructured.partition.pdf_image.ocr.process_file_with_ocr",
+    ) as mock_process_file_with_model:
+        partition(
+            example_doc_path("pdf/layout-parser-paper-fast.pdf"),
+            pdf_infer_table_structure=True,
+            strategy=PartitionStrategy.HI_RES,
+            infer_table_structure=None,
+        )
+        assert mock_process_file_with_model.call_args[1]["infer_table_structure"] is True
+
+
 def test_auto_partition_pdf_uses_pdf_infer_table_structure_argument():
     with patch(
         "unstructured.partition.pdf_image.ocr.process_file_with_ocr",
@@ -897,7 +923,7 @@ def test_auto_partition_raises_with_bad_type(request: FixtureRequest):
 
     with pytest.raises(
         UnsupportedFileFormatError,
-        match="Invalid file made-up.fake. The FileType.UNK file type is not supported in partiti",
+        match="Partitioning is not supported for the FileType.UNK file type.",
     ):
         partition(filename="made-up.fake", strategy=PartitionStrategy.HI_RES)
 
@@ -1037,26 +1063,6 @@ def test_auto_partition_forwards_metadata_filename_via_kwargs():
     assert all(e.metadata.filename == "much-more-interesting-name.txt" for e in elements)
 
 
-def test_auto_partition_warns_about_file_filename_deprecation(caplog: LogCaptureFixture):
-    file_path = example_doc_path("fake-text.txt")
-
-    with open(file_path, "rb") as f:
-        elements = partition(file=f, file_filename=file_path)
-
-    assert all(e.metadata.filename == "fake-text.txt" for e in elements)
-    assert caplog.records[0].levelname == "WARNING"
-    assert "The file_filename kwarg will be deprecated" in caplog.text
-
-
-def test_auto_partition_raises_when_both_file_filename_and_metadata_filename_args_are_used():
-    file_path = example_doc_path("fake-text.txt")
-    with open(file_path, "rb") as f:
-        file = io.BytesIO(f.read())
-
-    with pytest.raises(ValueError, match="Only one of metadata_filename and file_filename is spe"):
-        partition(file=file, file_filename=file_path, metadata_filename=file_path)
-
-
 # -- ocr_languages --------------------------------------------------------
 
 
@@ -1236,6 +1242,17 @@ def test_auto_partition_applies_the_correct_filetype_for_all_filetypes(
     )
 
 
+def test_detect_filetype_maps_file_to_bytes_io_when_spooled_temp_file_used(mocker):
+    detect_filetype_mock = MagicMock(return_value=FileType.JSON)
+    mocker.patch("unstructured.file_utils.filetype._FileTypeDetector", detect_filetype_mock)
+    with tempfile.SpooledTemporaryFile() as f:
+        f.write(b'{"text": Hello, world!}')
+        f.seek(0)
+        detect_filetype(file=f)
+    file_detection_context = detect_filetype_mock.file_type.call_args[0][0]
+    assert file_detection_context.text_head == '{"text": Hello, world!}'
+
+
 # -- .languages -----------------------------------------------------------
 
 
@@ -1308,7 +1325,7 @@ def expected_docx_elements():
         Title("These are a few of my favorite things:"),
         ListItem("Parrots"),
         ListItem("Hockey"),
-        Title("Analysis"),
+        Text("Analysis"),
         NarrativeText("This is my first thought. This is my second thought."),
         NarrativeText("This is my third thought."),
         Text("2023"),
