@@ -21,7 +21,7 @@ def _validate_language_code(language_code: str):
         )
 
 
-def translate_text(text, source_lang: Optional[str] = None, target_lang: str = "en") -> str:
+def translate_text(text: str, source_lang: Optional[str] = None, target_lang: str = "en") -> str:
     """Translates the foreign language text. If the source language is not specified, the
     function will attempt to detect it using langdetect.
 
@@ -52,6 +52,7 @@ def translate_text(text, source_lang: Optional[str] = None, target_lang: str = "
         return text
 
     model_name = _get_opus_mt_model_name(_source_lang, target_lang)
+    print(f"Using model: {model_name}")
 
     try:
         tokenizer = MarianTokenizer.from_pretrained(model_name)
@@ -79,7 +80,7 @@ def _translate_text(text, model, tokenizer):
     with warnings.catch_warnings():
         warnings.simplefilter("ignore")
         translated = model.generate(
-            **tokenizer([text], return_tensors="pt", padding="max_length", max_length=512),
+            **tokenizer([text], return_tensors="pt", padding=True, truncation=True),
         )
     return [tokenizer.decode(t, max_new_tokens=512, skip_special_tokens=True) for t in translated][
         0
