@@ -11,13 +11,13 @@ from typing import IO, TYPE_CHECKING, Any, Optional, cast
 
 import numpy as np
 import wrapt
-from pdfminer import psparser
-from pdfminer.layout import LTChar, LTContainer, LTImage, LTItem, LTTextBox
-from pdfminer.pdftypes import PDFObjRef
+from pdfminer.layout import LTContainer, LTImage, LTItem, LTTextBox
 from pdfminer.utils import open_filename
 from pi_heif import register_heif_opener
 from PIL import Image as PILImage
 from pypdf import PdfReader
+from unstructured_inference.inference.layout import DocumentLayout
+from unstructured_inference.inference.layoutelement import LayoutElement
 
 from unstructured.chunking import add_chunking_strategy
 from unstructured.cleaners.core import (
@@ -35,6 +35,7 @@ from unstructured.documents.elements import (
     ListItem,
     PageBreak,
     Text,
+    Title,
     process_metadata,
 )
 from unstructured.errors import PageCountExceededError
@@ -43,8 +44,10 @@ from unstructured.file_utils.model import FileType
 from unstructured.logger import logger, trace_logger
 from unstructured.nlp.patterns import PARAGRAPH_PATTERN
 from unstructured.partition.common.common import (
-    document_to_element_list,
+    add_element_metadata,
     exactly_one,
+    get_page_image_metadata,
+    normalize_layout_element,
     ocr_data_to_elements,
     spooled_to_bytes_io_if_needed,
 )
@@ -68,10 +71,16 @@ from unstructured.partition.pdf_image.pdf_image_utils import (
     save_elements,
 )
 from unstructured.partition.pdf_image.pdfminer_processing import (
+    check_annotations_within_element,
     clean_pdfminer_inner_elements,
+    get_links_in_element,
+    get_uris,
+    get_words_from_obj,
+    map_bbox_and_index,
     merge_inferred_with_extracted_layout,
 )
 from unstructured.partition.pdf_image.pdfminer_utils import (
+    PDFMinerConfig,
     open_pdfminer_pages_generator,
     rect_to_bbox,
 )
@@ -87,15 +96,18 @@ from unstructured.partition.utils.constants import (
     PartitionStrategy,
 )
 from unstructured.partition.utils.sorting import coord_has_valid_points, sort_page_elements
-from unstructured.patches.pdfminer import parse_keyword
-from unstructured.utils import requires_dependencies
+from unstructured.patches.pdfminer import patch_psparser
+from unstructured.utils import first, requires_dependencies
 
 if TYPE_CHECKING:
     pass
 
-# NOTE(alan): Patching this to fix a bug in pdfminer.six. Submitted this PR into pdfminer.six to fix
-# the bug: https://github.com/pdfminer/pdfminer.six/pull/885
-psparser.PSBaseParser._parse_keyword = parse_keyword  # type: ignore
+
+# Correct a bug that was introduced by a previous patch to
+# pdfminer.six, causing needless and unsuccessful repairing of PDFs
+# which were not actually broken.
+patch_psparser()
+
 
 RE_MULTISPACE_INCLUDING_NEWLINES = re.compile(pattern=r"\s+", flags=re.DOTALL)
 
@@ -133,6 +145,11 @@ def partition_pdf(
     starting_page_number: int = 1,
     extract_forms: bool = False,
     form_extraction_skip_tables: bool = True,
+    password: Optional[str] = None,
+    pdfminer_line_margin: Optional[float] = None,
+    pdfminer_char_margin: Optional[float] = None,
+    pdfminer_line_overlap: Optional[float] = None,
+    pdfminer_word_margin: Optional[float] = 0.185,
     **kwargs: Any,
 ) -> list[Element]:
     """Parses a pdf document into a list of interpreted elements.
@@ -191,12 +208,24 @@ def partition_pdf(
         (results in adding FormKeysValues elements to output).
     form_extraction_skip_tables
         Whether the form extraction logic should ignore regions designated as Tables.
+    pdfminer_line_margin
+        If two lines are close together they are considered to be part of the same paragraph.
+        The margin is specified relative to the height of a line.
+    pdfminer_char_margin
+        If two characters are closer together than this margin they are considered part of
+        the same line. The margin is specified relative to the width of the character.
+    pdfminer_line_overlap
+        If two characters have more overlap than this they are considered to be on the same line.
+        The overlap is specified relative to the minimum height of both characters.
+    pdfminer_word_margin
+        If two characters on the same line are further apart than this margin then they are
+        considered to be two separate words, and an intermediate space will be added for
+        readability. The margin is specified relative to the width of the character.
     """
 
     exactly_one(filename=filename, file=file)
 
     languages = check_language_args(languages or [], ocr_languages)
-
     return partition_pdf_or_image(
         filename=filename,
         file=file,
@@ -213,6 +242,11 @@ def partition_pdf(
         starting_page_number=starting_page_number,
         extract_forms=extract_forms,
         form_extraction_skip_tables=form_extraction_skip_tables,
+        password=password,
+        pdfminer_line_margin=pdfminer_line_margin,
+        pdfminer_char_margin=pdfminer_char_margin,
+        pdfminer_line_overlap=pdfminer_line_overlap,
+        pdfminer_word_margin=pdfminer_word_margin,
         **kwargs,
     )
 
@@ -234,6 +268,11 @@ def partition_pdf_or_image(
     starting_page_number: int = 1,
     extract_forms: bool = False,
     form_extraction_skip_tables: bool = True,
+    password: Optional[str] = None,
+    pdfminer_line_margin: Optional[float] = None,
+    pdfminer_char_margin: Optional[float] = None,
+    pdfminer_line_overlap: Optional[float] = None,
+    pdfminer_word_margin: Optional[float] = 0.185,
     **kwargs: Any,
 ) -> list[Element]:
     """Parses a pdf or image document into a list of interpreted elements."""
@@ -251,7 +290,12 @@ def partition_pdf_or_image(
     validate_strategy(strategy, is_image)
 
     last_modified = get_last_modified_date(filename) if filename else None
-
+    pdfminer_config = PDFMinerConfig(
+        line_margin=pdfminer_line_margin,
+        char_margin=pdfminer_char_margin,
+        line_overlap=pdfminer_line_overlap,
+        word_margin=pdfminer_word_margin,
+    )
     extracted_elements = []
     pdf_text_extractable = False
     if not is_image:
@@ -262,6 +306,8 @@ def partition_pdf_or_image(
                 languages=languages,
                 metadata_last_modified=metadata_last_modified or last_modified,
                 starting_page_number=starting_page_number,
+                password=password,
+                pdfminer_config=pdfminer_config,
                 **kwargs,
             )
             pdf_text_extractable = any(
@@ -311,6 +357,8 @@ def partition_pdf_or_image(
                 starting_page_number=starting_page_number,
                 extract_forms=extract_forms,
                 form_extraction_skip_tables=form_extraction_skip_tables,
+                password=password,
+                pdfminer_config=pdfminer_config,
                 **kwargs,
             )
             out_elements = _process_uncategorized_text_elements(elements)
@@ -336,6 +384,7 @@ def partition_pdf_or_image(
                 is_image=is_image,
                 metadata_last_modified=metadata_last_modified or last_modified,
                 starting_page_number=starting_page_number,
+                password=password,
                 **kwargs,
             )
             out_elements = _process_uncategorized_text_elements(elements)
@@ -349,6 +398,8 @@ def extractable_elements(
     languages: Optional[list[str]] = None,
     metadata_last_modified: Optional[str] = None,
     starting_page_number: int = 1,
+    password: Optional[str] = None,
+    pdfminer_config: Optional[PDFMinerConfig] = None,
     **kwargs: Any,
 ) -> list[list[Element]]:
     if isinstance(file, bytes):
@@ -359,6 +410,8 @@ def extractable_elements(
         languages=languages,
         metadata_last_modified=metadata_last_modified,
         starting_page_number=starting_page_number,
+        password=password,
+        pdfminer_config=pdfminer_config,
         **kwargs,
     )
 
@@ -369,12 +422,14 @@ def _partition_pdf_with_pdfminer(
     languages: list[str],
     metadata_last_modified: Optional[str],
     starting_page_number: int = 1,
+    password: Optional[str] = None,
+    pdfminer_config: Optional[PDFMinerConfig] = None,
     **kwargs: Any,
 ) -> list[list[Element]]:
     """Partitions a PDF using PDFMiner instead of using a layoutmodel. Used for faster
     processing or detectron2 is not available.
 
-    Implementation is based on the `extract_text` implemenation in pdfminer.six, but
+    Implementation is based on the `extract_text` implementation in pdfminer.six, but
     modified to support tracking page numbers and working with file-like objects.
 
     ref: https://github.com/pdfminer/pdfminer.six/blob/master/pdfminer/high_level.py
@@ -392,6 +447,8 @@ def _partition_pdf_with_pdfminer(
                 languages=languages,
                 metadata_last_modified=metadata_last_modified,
                 starting_page_number=starting_page_number,
+                password=password,
+                pdfminer_config=pdfminer_config,
                 **kwargs,
             )
 
@@ -402,6 +459,8 @@ def _partition_pdf_with_pdfminer(
             languages=languages,
             metadata_last_modified=metadata_last_modified,
             starting_page_number=starting_page_number,
+            password=password,
+            pdfminer_config=pdfminer_config,
             **kwargs,
         )
 
@@ -416,6 +475,8 @@ def _process_pdfminer_pages(
     metadata_last_modified: Optional[str],
     annotation_threshold: Optional[float] = env_config.PDF_ANNOTATION_THRESHOLD,
     starting_page_number: int = 1,
+    password: Optional[str] = None,
+    pdfminer_config: Optional[PDFMinerConfig] = None,
     **kwargs,
 ) -> list[list[Element]]:
     """Uses PDFMiner to split a document into pages and process them."""
@@ -423,7 +484,8 @@ def _process_pdfminer_pages(
     elements = []
 
     for page_number, (page, page_layout) in enumerate(
-        open_pdfminer_pages_generator(fp), start=starting_page_number
+        open_pdfminer_pages_generator(fp, password=password, pdfminer_config=pdfminer_config),
+        start=starting_page_number,
     ):
         width, height = page_layout.width, page_layout.height
 
@@ -450,7 +512,7 @@ def _process_pdfminer_pages(
                     page_number,
                     annotation_threshold,
                 )
-                _, words = get_word_bounding_box_from_element(obj, height)
+                _, words = get_words_from_obj(obj, height)
                 for annot in annotations_within_element:
                     urls_metadata.append(map_bbox_and_index(words, annot))
 
@@ -545,6 +607,8 @@ def _partition_pdf_or_image_local(
     extract_forms: bool = False,
     form_extraction_skip_tables: bool = True,
     pdf_hi_res_max_pages: Optional[int] = None,
+    password: Optional[str] = None,
+    pdfminer_config: Optional[PDFMinerConfig] = None,
     **kwargs: Any,
 ) -> list[Element]:
     """Partition using package installed locally"""
@@ -581,12 +645,18 @@ def _partition_pdf_or_image_local(
             is_image=is_image,
             model_name=hi_res_model_name,
             pdf_image_dpi=pdf_image_dpi,
+            password=password,
         )
 
-        extracted_layout = (
-            process_file_with_pdfminer(filename=filename, dpi=pdf_image_dpi)
+        extracted_layout, layouts_links = (
+            process_file_with_pdfminer(
+                filename=filename,
+                dpi=pdf_image_dpi,
+                password=password,
+                pdfminer_config=pdfminer_config,
+            )
             if pdf_text_extractable
-            else []
+            else ([], [])
         )
 
         if analysis:
@@ -604,7 +674,7 @@ def _partition_pdf_or_image_local(
                     model_name=hi_res_model_name,
                 )
                 extracted_layout_dumper = ExtractedLayoutDumper(
-                    layout=extracted_layout,
+                    layout=[layout.as_list() for layout in extracted_layout],
                 )
                 ocr_layout_dumper = OCRLayoutDumper()
         # NOTE(christine): merged_document_layout = extracted_layout + inferred_layout
@@ -624,6 +694,7 @@ def _partition_pdf_or_image_local(
             ocr_mode=ocr_mode,
             pdf_image_dpi=pdf_image_dpi,
             ocr_layout_dumper=ocr_layout_dumper,
+            password=password,
         )
     else:
         inferred_document_layout = process_data_with_model(
@@ -631,13 +702,18 @@ def _partition_pdf_or_image_local(
             is_image=is_image,
             model_name=hi_res_model_name,
             pdf_image_dpi=pdf_image_dpi,
+            password=password,
         )
 
         if hasattr(file, "seek"):
             file.seek(0)
 
-        extracted_layout = (
-            process_data_with_pdfminer(file=file, dpi=pdf_image_dpi) if pdf_text_extractable else []
+        extracted_layout, layouts_links = (
+            process_data_with_pdfminer(
+                file=file, dpi=pdf_image_dpi, password=password, pdfminer_config=pdfminer_config
+            )
+            if pdf_text_extractable
+            else ([], [])
         )
 
         if analysis:
@@ -654,7 +730,7 @@ def _partition_pdf_or_image_local(
                     model_name=hi_res_model_name,
                 )
                 extracted_layout_dumper = ExtractedLayoutDumper(
-                    layout=extracted_layout,
+                    layout=[layout.as_list() for layout in extracted_layout],
                 )
                 ocr_layout_dumper = OCRLayoutDumper()
 
@@ -677,8 +753,10 @@ def _partition_pdf_or_image_local(
             ocr_mode=ocr_mode,
             pdf_image_dpi=pdf_image_dpi,
             ocr_layout_dumper=ocr_layout_dumper,
+            password=password,
         )
 
+    # vectorization of the data structure ends here
     final_document_layout = clean_pdfminer_inner_elements(final_document_layout)
 
     for page in final_document_layout.pages:
@@ -696,6 +774,7 @@ def _partition_pdf_or_image_local(
         infer_list_items=False,
         languages=languages,
         starting_page_number=starting_page_number,
+        layouts_links=layouts_links,
         **kwargs,
     )
 
@@ -822,6 +901,7 @@ def _partition_pdf_or_image_with_ocr(
     is_image: bool = False,
     metadata_last_modified: Optional[str] = None,
     starting_page_number: int = 1,
+    password: Optional[str] = None,
     **kwargs: Any,
 ):
     """Partitions an image or PDF using OCR. For PDFs, each page is converted
@@ -846,7 +926,7 @@ def _partition_pdf_or_image_with_ocr(
             elements.extend(page_elements)
     else:
         for page_number, image in enumerate(
-            convert_pdf_to_images(filename, file), start=starting_page_number
+            convert_pdf_to_images(filename, file, password=password), start=starting_page_number
         ):
             page_elements = _partition_pdf_or_image_with_ocr_from_image(
                 image=image,
@@ -891,8 +971,10 @@ def _partition_pdf_or_image_with_ocr_from_image(
         languages=languages,
     )
 
+    # NOTE (yao): elements for a document is still stored as a list therefore at this step we have
+    # to convert the vector data structured ocr_data into a list
     page_elements = ocr_data_to_elements(
-        ocr_data,
+        ocr_data.as_list(),
         image_size=image.size,
         common_metadata=metadata,
     )
@@ -1076,323 +1158,115 @@ def check_coords_within_boundary(
     return x_within_boundary and y_within_boundary
 
 
-def get_uris(
-    annots: PDFObjRef | list[PDFObjRef],
-    height: float,
-    coordinate_system: PixelSpace | PointSpace,
-    page_number: int,
-) -> list[dict[str, Any]]:
-    """
-    Extracts URI annotations from a single or a list of PDF object references on a specific page.
-    The type of annots (list or not) depends on the pdf formatting. The function detectes the type
-    of annots and then pass on to get_uris_from_annots function as a list.
+def document_to_element_list(
+    document: DocumentLayout,
+    sortable: bool = False,
+    include_page_breaks: bool = False,
+    last_modification_date: Optional[str] = None,
+    infer_list_items: bool = True,
+    source_format: Optional[str] = None,
+    detection_origin: Optional[str] = None,
+    sort_mode: str = SORT_MODE_XY_CUT,
+    languages: Optional[list[str]] = None,
+    starting_page_number: int = 1,
+    layouts_links: Optional[list[list]] = None,
+    **kwargs: Any,
+) -> list[Element]:
+    """Converts a DocumentLayout object to a list of unstructured elements."""
+    elements: list[Element] = []
 
-    Args:
-        annots (PDFObjRef | list[PDFObjRef]): A single or a list of PDF object references
-            representing annotations on the page.
-        height (float): The height of the page in the specified coordinate system.
-        coordinate_system (PixelSpace | PointSpace): The coordinate system used to represent
-            the annotations' coordinates.
-        page_number (int): The page number from which to extract annotations.
+    num_pages = len(document.pages)
+    for page_number, page in enumerate(document.pages, start=starting_page_number):
+        page_elements: list[Element] = []
 
-    Returns:
-        list[dict]: A list of dictionaries, each containing information about a URI annotation,
-        including its coordinates, bounding box, type, URI link, and page number.
-    """
-    if isinstance(annots, list):
-        return get_uris_from_annots(annots, height, coordinate_system, page_number)
-    resolved_annots = annots.resolve()
-    if resolved_annots is None:
-        return []
-    return get_uris_from_annots(resolved_annots, height, coordinate_system, page_number)
+        page_image_metadata = get_page_image_metadata(page)
+        image_format = page_image_metadata.get("format")
+        image_width = page_image_metadata.get("width")
+        image_height = page_image_metadata.get("height")
 
+        translation_mapping: list[tuple["LayoutElement", Element]] = []
 
-def get_uris_from_annots(
-    annots: list[PDFObjRef],
-    height: int | float,
-    coordinate_system: PixelSpace | PointSpace,
-    page_number: int,
-) -> list[dict[str, Any]]:
-    """
-    Extracts URI annotations from a list of PDF object references.
-
-    Args:
-        annots (list[PDFObjRef]): A list of PDF object references representing annotations on
-            a page.
-        height (int | float): The height of the page in the specified coordinate system.
-        coordinate_system (PixelSpace | PointSpace): The coordinate system used to represent
-            the annotations' coordinates.
-        page_number (int): The page number from which to extract annotations.
-
-    Returns:
-        list[dict]: A list of dictionaries, each containing information about a URI annotation,
-        including its coordinates, bounding box, type, URI link, and page number.
-    """
-    annotation_list = []
-    for annotation in annots:
-        # Check annotation is valid for extraction
-        annotation_dict = try_resolve(annotation)
-        if not isinstance(annotation_dict, dict):
-            continue
-        subtype = annotation_dict.get("Subtype", None)
-        if not subtype or isinstance(subtype, PDFObjRef) or str(subtype) != "/'Link'":
-            continue
-        # Extract bounding box and update coordinates
-        rect = annotation_dict.get("Rect", None)
-        if not rect or isinstance(rect, PDFObjRef) or len(rect) != 4:
-            continue
-        x1, y1, x2, y2 = rect_to_bbox(rect, height)
-        points = ((x1, y1), (x1, y2), (x2, y2), (x2, y1))
-        coordinates_metadata = CoordinatesMetadata(
-            points=points,
-            system=coordinate_system,
+        links = (
+            layouts_links[page_number - starting_page_number]
+            if layouts_links and layouts_links[0]
+            else None
         )
-        # Extract type
-        if "A" not in annotation_dict:
-            continue
-        uri_dict = try_resolve(annotation_dict["A"])
-        if not isinstance(uri_dict, dict):
-            continue
-        uri_type = None
-        if "S" in uri_dict and not isinstance(uri_dict["S"], PDFObjRef):
-            uri_type = str(uri_dict["S"])
-        # Extract URI link
-        uri = None
-        try:
-            if uri_type == "/'URI'":
-                uri = try_resolve(try_resolve(uri_dict["URI"])).decode("utf-8")
-            if uri_type == "/'GoTo'":
-                uri = try_resolve(try_resolve(uri_dict["D"])).decode("utf-8")
-        except Exception:
-            pass
 
-        annotation_list.append(
-            {
-                "coordinates": coordinates_metadata,
-                "bbox": (x1, y1, x2, y2),
-                "type": uri_type,
-                "uri": uri,
-                "page_number": page_number,
-            },
-        )
-    return annotation_list
-
-
-def try_resolve(annot: PDFObjRef):
-    """
-    Attempt to resolve a PDF object reference. If successful, returns the resolved object;
-    otherwise, returns the original reference.
-    """
-    try:
-        return annot.resolve()
-    except Exception:
-        return annot
-
-
-def calculate_intersection_area(
-    bbox1: tuple[float, float, float, float],
-    bbox2: tuple[float, float, float, float],
-) -> float:
-    """
-    Calculate the area of intersection between two bounding boxes.
-
-    Args:
-        bbox1 (tuple[float, float, float, float]): The coordinates of the first bounding box
-            in the format (x1, y1, x2, y2).
-        bbox2 (tuple[float, float, float, float]): The coordinates of the second bounding box
-            in the format (x1, y1, x2, y2).
-
-    Returns:
-        float: The area of intersection between the two bounding boxes. If there is no
-        intersection, the function returns 0.0.
-    """
-    x1_1, y1_1, x2_1, y2_1 = bbox1
-    x1_2, y1_2, x2_2, y2_2 = bbox2
-
-    x_intersection = max(x1_1, x1_2)
-    y_intersection = max(y1_1, y1_2)
-    x2_intersection = min(x2_1, x2_2)
-    y2_intersection = min(y2_1, y2_2)
-
-    if x_intersection < x2_intersection and y_intersection < y2_intersection:
-        intersection_area = calculate_bbox_area(
-            (x_intersection, y_intersection, x2_intersection, y2_intersection),
-        )
-        return intersection_area
-    else:
-        return 0.0
-
-
-def calculate_bbox_area(bbox: tuple[float, float, float, float]) -> float:
-    """
-    Calculate the area of a bounding box.
-
-    Args:
-        bbox (tuple[float, float, float, float]): The coordinates of the bounding box
-            in the format (x1, y1, x2, y2).
-
-    Returns:
-        float: The area of the bounding box, computed as the product of its width and height.
-    """
-    x1, y1, x2, y2 = bbox
-    area = (x2 - x1) * (y2 - y1)
-    return area
-
-
-def check_annotations_within_element(
-    annotation_list: list[dict[str, Any]],
-    element_bbox: tuple[float, float, float, float],
-    page_number: int,
-    annotation_threshold: float,
-) -> list[dict[str, Any]]:
-    """
-    Filter annotations that are within or highly overlap with a specified element on a page.
-
-    Args:
-        annotation_list (list[dict[str,Any]]): A list of dictionaries, each containing information
-            about an annotation.
-        element_bbox (tuple[float, float, float, float]): The bounding box coordinates of the
-            specified element in the bbox format (x1, y1, x2, y2).
-        page_number (int): The page number to which the annotations and element belong.
-        annotation_threshold (float, optional): The threshold value (between 0.0 and 1.0)
-            that determines the minimum overlap required for an annotation to be considered
-            within the element. Default is 0.9.
-
-    Returns:
-        list[dict[str,Any]]: A list of dictionaries containing information about annotations
-        that are within or highly overlap with the specified element on the given page, based on
-        the specified threshold.
-    """
-    annotations_within_element = []
-    for annotation in annotation_list:
-        if annotation["page_number"] == page_number:
-            annotation_bbox_size = calculate_bbox_area(annotation["bbox"])
-            if annotation_bbox_size and (
-                calculate_intersection_area(element_bbox, annotation["bbox"]) / annotation_bbox_size
-                > annotation_threshold
+        for layout_element in page.elements:
+            if (
+                image_width
+                and image_height
+                and getattr(layout_element.bbox, "x1") not in (None, np.nan)
             ):
-                annotations_within_element.append(annotation)
-    return annotations_within_element
+                coordinate_system = PixelSpace(width=image_width, height=image_height)
+            else:
+                coordinate_system = None
 
+            element = normalize_layout_element(
+                layout_element,
+                coordinate_system=coordinate_system,
+                infer_list_items=infer_list_items,
+                source_format=source_format if source_format else "html",
+            )
+            if isinstance(element, list):
+                for el in element:
+                    if last_modification_date:
+                        el.metadata.last_modified = last_modification_date
+                    el.metadata.page_number = page_number
+                page_elements.extend(element)
+                translation_mapping.extend([(layout_element, el) for el in element])
+                continue
+            else:
 
-def get_word_bounding_box_from_element(
-    obj: LTTextBox,
-    height: float,
-) -> tuple[list[LTChar], list[dict[str, Any]]]:
-    """
-    Extracts characters and word bounding boxes from a PDF text element.
+                element.metadata.links = (
+                    get_links_in_element(links, layout_element.bbox) if links else []
+                )
 
-    Args:
-        obj (LTTextBox): The PDF text element from which to extract characters and words.
-        height (float): The height of the page in the specified coordinate system.
+                if last_modification_date:
+                    element.metadata.last_modified = last_modification_date
+                element.metadata.text_as_html = getattr(layout_element, "text_as_html", None)
+                element.metadata.table_as_cells = getattr(layout_element, "table_as_cells", None)
 
-    Returns:
-        tuple[list[LTChar], list[dict[str,Any]]]: A tuple containing two lists:
-            - list[LTChar]: A list of LTChar objects representing individual characters.
-            - list[dict[str,Any]]]: A list of dictionaries, each containing information about
-                a word, including its text, bounding box, and start index in the element's text.
-    """
-    characters = []
-    words = []
-    text_len = 0
+                if (isinstance(element, Title) and element.metadata.category_depth is None) and any(
+                    getattr(el, "type", "") in ["Headline", "Subheadline"] for el in page.elements
+                ):
+                    element.metadata.category_depth = 0
 
-    for text_line in obj:
-        word = ""
-        x1, y1, x2, y2 = None, None, None, None
-        start_index = 0
-        for index, character in enumerate(text_line):
-            if isinstance(character, LTChar):
-                characters.append(character)
-                char = character.get_text()
+                page_elements.append(element)
+                translation_mapping.append((layout_element, element))
+            coordinates = (
+                element.metadata.coordinates.points if element.metadata.coordinates else None
+            )
 
-                if word and not char.strip():
-                    words.append(
-                        {"text": word, "bbox": (x1, y1, x2, y2), "start_index": start_index},
-                    )
-                    word = ""
-                    continue
+            el_image_path = (
+                layout_element.image_path if hasattr(layout_element, "image_path") else None
+            )
 
-                # TODO(klaijan) - isalnum() only works with A-Z, a-z and 0-9
-                # will need to switch to some pattern matching once we support more languages
-                if not word:
-                    isalnum = char.isalnum()
-                if word and char.isalnum() != isalnum:
-                    isalnum = char.isalnum()
-                    words.append(
-                        {"text": word, "bbox": (x1, y1, x2, y2), "start_index": start_index},
-                    )
-                    word = ""
+            add_element_metadata(
+                element,
+                page_number=page_number,
+                filetype=image_format,
+                coordinates=coordinates,
+                coordinate_system=coordinate_system,
+                category_depth=element.metadata.category_depth,
+                image_path=el_image_path,
+                detection_origin=detection_origin,
+                languages=languages,
+                **kwargs,
+            )
 
-                if len(word) == 0:
-                    start_index = text_len + index
-                    x1 = character.x0
-                    y2 = height - character.y0
-                    x2 = character.x1
-                    y1 = height - character.y1
-                else:
-                    x2 = character.x1
-                    y2 = height - character.y0
+        for layout_element, element in translation_mapping:
+            if hasattr(layout_element, "parent") and layout_element.parent is not None:
+                element_parent = first(
+                    (el for l_el, el in translation_mapping if l_el is layout_element.parent),
+                )
+                element.metadata.parent_id = element_parent.id
+        sorted_page_elements = page_elements
+        if sortable and sort_mode != SORT_MODE_DONT:
+            sorted_page_elements = sort_page_elements(page_elements, sort_mode)
 
-                word += char
-        text_len += len(text_line)
-    return characters, words
+        if include_page_breaks and page_number < num_pages + starting_page_number:
+            sorted_page_elements.append(PageBreak(text=""))
+        elements.extend(sorted_page_elements)
 
-
-def map_bbox_and_index(words: list[dict[str, Any]], annot: dict[str, Any]):
-    """
-    Maps a bounding box annotation to the corresponding text and start index within a list of words.
-
-    Args:
-        words (list[dict[str,Any]]): A list of dictionaries, each containing information about
-            a word, including its text, bounding box, and start index.
-        annot (dict[str,Any]): The annotation dictionary to be mapped, which will be updated with
-        "text" and "start_index" fields.
-
-    Returns:
-        dict: The updated annotation dictionary with "text" representing the mapped text and
-            "start_index" representing the start index of the mapped text in the list of words.
-    """
-    if len(words) == 0:
-        annot["text"] = ""
-        annot["start_index"] = -1
-        return annot
-    distance_from_bbox_start = np.sqrt(
-        (annot["bbox"][0] - np.array([word["bbox"][0] for word in words])) ** 2
-        + (annot["bbox"][1] - np.array([word["bbox"][1] for word in words])) ** 2,
-    )
-    distance_from_bbox_end = np.sqrt(
-        (annot["bbox"][2] - np.array([word["bbox"][2] for word in words])) ** 2
-        + (annot["bbox"][3] - np.array([word["bbox"][3] for word in words])) ** 2,
-    )
-    closest_start = try_argmin(distance_from_bbox_start)
-    closest_end = try_argmin(distance_from_bbox_end)
-
-    # NOTE(klaijan) - get the word from closest start only if the end index comes after start index
-    text = ""
-    if closest_end >= closest_start:
-        for _ in range(closest_start, closest_end + 1):
-            text += " "
-            text += words[_]["text"]
-    else:
-        text = words[closest_start]["text"]
-
-    annot["text"] = text.strip()
-    annot["start_index"] = words[closest_start]["start_index"]
-    return annot
-
-
-def try_argmin(array: np.ndarray) -> int:
-    """
-    Attempt to find the index of the minimum value in a NumPy array.
-
-    Args:
-        array (np.ndarray): The NumPy array in which to find the minimum value's index.
-
-    Returns:
-        int: The index of the minimum value in the array. If the array is empty or an
-        IndexError occurs, it returns -1.
-    """
-    try:
-        return int(np.argmin(array))
-    except IndexError:
-        return -1
+    return elements
