@@ -4,6 +4,7 @@ import os
 import tempfile
 from typing import IO, TYPE_CHECKING, Any, List, Optional, cast
 
+import numpy as np
 import pdf2image
 
 # NOTE(yuming): Rename PIL.Image to avoid conflict with
@@ -13,17 +14,21 @@ from PIL import ImageSequence
 
 from unstructured.documents.elements import ElementType
 from unstructured.metrics.table.table_formats import SimpleTableCell
-from unstructured.partition.pdf_image.analysis.bbox_visualisation import OCRLayoutDrawer
-from unstructured.partition.pdf_image.pdf_image_utils import pad_element_bboxes, valid_text
+from unstructured.partition.pdf_image.analysis.layout_dump import OCRLayoutDumper
+from unstructured.partition.pdf_image.pdf_image_utils import valid_text
+from unstructured.partition.pdf_image.pdfminer_processing import (
+    aggregate_embedded_text_by_block,
+    bboxes1_is_almost_subregion_of_bboxes2,
+)
 from unstructured.partition.utils.config import env_config
-from unstructured.partition.utils.constants import OCR_AGENT_TESSERACT, OCRMode
+from unstructured.partition.utils.constants import OCRMode
 from unstructured.partition.utils.ocr_models.ocr_interface import OCRAgent
 from unstructured.utils import requires_dependencies
 
 if TYPE_CHECKING:
-    from unstructured_inference.inference.elements import TextRegion
+    from unstructured_inference.inference.elements import TextRegion, TextRegions
     from unstructured_inference.inference.layout import DocumentLayout, PageLayout
-    from unstructured_inference.inference.layoutelement import LayoutElement
+    from unstructured_inference.inference.layoutelement import LayoutElement, LayoutElements
     from unstructured_inference.models.tables import UnstructuredTableTransformerModel
 
 
@@ -36,7 +41,8 @@ def process_data_with_ocr(
     ocr_languages: str = "eng",
     ocr_mode: str = OCRMode.FULL_PAGE.value,
     pdf_image_dpi: int = 200,
-    ocr_drawer: Optional[OCRLayoutDrawer] = None,
+    ocr_layout_dumper: Optional[OCRLayoutDumper] = None,
+    password: Optional[str] = None,
 ) -> "DocumentLayout":
     """
     Process OCR data from a given data and supplement the output DocumentLayout
@@ -62,19 +68,20 @@ def process_data_with_ocr(
 
     - pdf_image_dpi (int, optional): DPI (dots per inch) for processing PDF images. Defaults to 200.
 
+    - ocr_layout_dumper (OCRLayoutDumper, optional): The OCR layout dumper to save the OCR layout.
+
     Returns:
         DocumentLayout: The merged layout information obtained after OCR processing.
     """
-    file_name = ""
-    with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
-        data_bytes = data if isinstance(data, bytes) else data.read()
-        tmp_file.write(data_bytes)
-        tmp_file.flush()
-        file_name = tmp_file.name
+    data_bytes = data if isinstance(data, bytes) else data.read()
+
+    with tempfile.TemporaryDirectory() as tmp_dir_path:
+        tmp_file_path = os.path.join(tmp_dir_path, "tmp_file")
+        with open(tmp_file_path, "wb") as tmp_file:
+            tmp_file.write(data_bytes)
 
-    try:
         merged_layouts = process_file_with_ocr(
-            filename=file_name,
+            filename=tmp_file_path,
             out_layout=out_layout,
             extracted_layout=extracted_layout,
             is_image=is_image,
@@ -82,11 +89,9 @@ def process_data_with_ocr(
             ocr_languages=ocr_languages,
             ocr_mode=ocr_mode,
             pdf_image_dpi=pdf_image_dpi,
-            ocr_drawer=ocr_drawer,
+            ocr_layout_dumper=ocr_layout_dumper,
+            password=password,
         )
-    finally:
-        if os.path.isfile(file_name):
-            os.remove(file_name)
 
     return merged_layouts
 
@@ -95,13 +100,14 @@ def process_data_with_ocr(
 def process_file_with_ocr(
     filename: str,
     out_layout: "DocumentLayout",
-    extracted_layout: List[List["TextRegion"]],
+    extracted_layout: List[TextRegions],
     is_image: bool = False,
     infer_table_structure: bool = False,
     ocr_languages: str = "eng",
     ocr_mode: str = OCRMode.FULL_PAGE.value,
     pdf_image_dpi: int = 200,
-    ocr_drawer: Optional[OCRLayoutDrawer] = None,
+    ocr_layout_dumper: Optional[OCRLayoutDumper] = None,
+    password: Optional[str] = None,
 ) -> "DocumentLayout":
     """
     Process OCR data from a given file and supplement the output DocumentLayout
@@ -112,6 +118,9 @@ def process_file_with_ocr(
 
     - out_layout (DocumentLayout): The output layout from unstructured-inference.
 
+    - extracted_layout (List[TextRegions]): a list of text regions extracted by pdfminer, one for
+      each page
+
     - is_image (bool, optional): Indicates if the input data is an image (True) or not (False).
         Defaults to False.
 
@@ -148,7 +157,7 @@ def process_file_with_ocr(
                         ocr_languages=ocr_languages,
                         ocr_mode=ocr_mode,
                         extracted_regions=extracted_regions,
-                        ocr_drawer=ocr_drawer,
+                        ocr_layout_dumper=ocr_layout_dumper,
                     )
                     merged_page_layouts.append(merged_page_layout)
                 return DocumentLayout.from_pages(merged_page_layouts)
@@ -159,6 +168,7 @@ def process_file_with_ocr(
                     dpi=pdf_image_dpi,
                     output_folder=temp_dir,
                     paths_only=True,
+                    userpw=password or "",
                 )
                 image_paths = cast(List[str], _image_paths)
                 for i, image_path in enumerate(image_paths):
@@ -171,7 +181,7 @@ def process_file_with_ocr(
                             ocr_languages=ocr_languages,
                             ocr_mode=ocr_mode,
                             extracted_regions=extracted_regions,
-                            ocr_drawer=ocr_drawer,
+                            ocr_layout_dumper=ocr_layout_dumper,
                         )
                         merged_page_layouts.append(merged_page_layout)
                 return DocumentLayout.from_pages(merged_page_layouts)
@@ -189,8 +199,8 @@ def supplement_page_layout_with_ocr(
     infer_table_structure: bool = False,
     ocr_languages: str = "eng",
     ocr_mode: str = OCRMode.FULL_PAGE.value,
-    extracted_regions: Optional[List["TextRegion"]] = None,
-    ocr_drawer: Optional[OCRLayoutDrawer] = None,
+    extracted_regions: Optional[TextRegions] = None,
+    ocr_layout_dumper: Optional[OCRLayoutDumper] = None,
 ) -> "PageLayout":
     """
     Supplement an PageLayout with OCR results depending on OCR mode.
@@ -200,38 +210,34 @@ def supplement_page_layout_with_ocr(
     with no text and add text from OCR to each element.
     """
 
-    ocr_agent = OCRAgent.get_agent()
+    ocr_agent = OCRAgent.get_agent(language=ocr_languages)
     if ocr_mode == OCRMode.FULL_PAGE.value:
-        ocr_layout = ocr_agent.get_layout_from_image(
-            image,
-            ocr_languages=ocr_languages,
-        )
-        if ocr_drawer:
-            ocr_drawer.add_ocred_page(ocr_layout)
-        page_layout.elements[:] = merge_out_layout_with_ocr_layout(
-            out_layout=cast(List["LayoutElement"], page_layout.elements),
+        ocr_layout = ocr_agent.get_layout_from_image(image)
+        if ocr_layout_dumper:
+            ocr_layout_dumper.add_ocred_page(ocr_layout.as_list())
+        page_layout.elements_array = merge_out_layout_with_ocr_layout(
+            out_layout=page_layout.elements_array,
             ocr_layout=ocr_layout,
         )
     elif ocr_mode == OCRMode.INDIVIDUAL_BLOCKS.value:
-        for element in page_layout.elements:
-            if not element.text:
-                padding = env_config.IMAGE_CROP_PAD
-                padded_element = pad_element_bboxes(element, padding=padding)
-                cropped_image = image.crop(
-                    (
-                        padded_element.bbox.x1,
-                        padded_element.bbox.y1,
-                        padded_element.bbox.x2,
-                        padded_element.bbox.y2,
-                    ),
-                )
-                # Note(yuming): instead of getting OCR layout, we just need
-                # the text extraced from OCR for individual elements
-                text_from_ocr = ocr_agent.get_text_from_image(
-                    cropped_image,
-                    ocr_languages=ocr_languages,
-                )
-                element.text = text_from_ocr
+        # individual block mode still keeps using the list data structure for elements instead of
+        # the vectorized page_layout.elements_array data structure
+        for i, text in enumerate(page_layout.elements_array.texts):
+            if text:
+                continue
+            padding = env_config.IMAGE_CROP_PAD
+            cropped_image = image.crop(
+                (
+                    page_layout.elements_array.x1[i] - padding,
+                    page_layout.elements_array.y1[i] - padding,
+                    page_layout.elements_array.x2[i] + padding,
+                    page_layout.elements_array.y2[i] + padding,
+                ),
+            )
+            # Note(yuming): instead of getting OCR layout, we just need
+            # the text extraced from OCR for individual elements
+            text_from_ocr = ocr_agent.get_text_from_image(cropped_image)
+            page_layout.elements_array.texts[i] = text_from_ocr
     else:
         raise ValueError(
             "Invalid OCR mode. Parameter `ocr_mode` "
@@ -246,26 +252,25 @@ def supplement_page_layout_with_ocr(
         if tables.tables_agent is None:
             raise RuntimeError("Unable to load table extraction agent.")
 
-        page_layout.elements[:] = supplement_element_with_table_extraction(
-            elements=cast(List["LayoutElement"], page_layout.elements),
+        page_layout.elements_array = supplement_element_with_table_extraction(
+            elements=page_layout.elements_array,
             image=image,
             tables_agent=tables.tables_agent,
-            ocr_languages=ocr_languages,
             ocr_agent=ocr_agent,
             extracted_regions=extracted_regions,
         )
+    page_layout.elements = page_layout.elements_array.as_list()
 
     return page_layout
 
 
 @requires_dependencies("unstructured_inference")
 def supplement_element_with_table_extraction(
-    elements: List["LayoutElement"],
+    elements: LayoutElements,
     image: PILImage.Image,
     tables_agent: "UnstructuredTableTransformerModel",
-    ocr_languages: str = "eng",
-    ocr_agent: OCRAgent = OCRAgent.get_instance(OCR_AGENT_TESSERACT),
-    extracted_regions: Optional[List["TextRegion"]] = None,
+    ocr_agent,
+    extracted_regions: Optional[TextRegions] = None,
 ) -> List["LayoutElement"]:
     """Supplement the existing layout with table extraction. Any Table elements
     that are extracted will have a metadata fields "text_as_html" where
@@ -274,24 +279,26 @@ def supplement_element_with_table_extraction(
     """
     from unstructured_inference.models.tables import cells_to_html
 
-    table_elements = [el for el in elements if el.type == ElementType.TABLE]
-    for element in table_elements:
-        padding = env_config.TABLE_IMAGE_CROP_PAD
-        padded_element = pad_element_bboxes(element, padding=padding)
+    table_id = {v: k for k, v in elements.element_class_id_map.items()}.get(ElementType.TABLE)
+    if table_id is None:
+        # no table found in this page
+        return elements
+
+    table_ele_indices = np.where(elements.element_class_ids == table_id)[0]
+    table_elements = elements.slice(table_ele_indices)
+    padding = env_config.TABLE_IMAGE_CROP_PAD
+    for i, element_coords in enumerate(table_elements.element_coords):
         cropped_image = image.crop(
             (
-                padded_element.bbox.x1,
-                padded_element.bbox.y1,
-                padded_element.bbox.x2,
-                padded_element.bbox.y2,
+                element_coords[0] - padding,
+                element_coords[1] - padding,
+                element_coords[2] + padding,
+                element_coords[3] + padding,
             ),
         )
         table_tokens = get_table_tokens(
             table_element_image=cropped_image,
-            ocr_languages=ocr_languages,
             ocr_agent=ocr_agent,
-            extracted_regions=extracted_regions,
-            table_element=padded_element,
         )
         tatr_cells = tables_agent.predict(
             cropped_image, ocr_tokens=table_tokens, result_format="cells"
@@ -299,62 +306,52 @@ def supplement_element_with_table_extraction(
 
         # NOTE(christine): `tatr_cells == ""` means that the table was not recognized
         text_as_html = "" if tatr_cells == "" else cells_to_html(tatr_cells)
-        element.text_as_html = text_as_html
+        elements.text_as_html[table_ele_indices[i]] = text_as_html
 
         if env_config.EXTRACT_TABLE_AS_CELLS:
             simple_table_cells = [
                 SimpleTableCell.from_table_transformer_cell(cell).to_dict() for cell in tatr_cells
             ]
-            element.table_as_cells = simple_table_cells
+            elements.table_as_cells[table_ele_indices[i]] = simple_table_cells
 
     return elements
 
 
 def get_table_tokens(
     table_element_image: PILImage.Image,
-    ocr_languages: str = "eng",
-    ocr_agent: OCRAgent = OCRAgent.get_instance(OCR_AGENT_TESSERACT),
-    extracted_regions: Optional[List["TextRegion"]] = None,
-    table_element: Optional["LayoutElement"] = None,
+    ocr_agent: OCRAgent,
 ) -> List[dict[str, Any]]:
     """Get OCR tokens from either paddleocr or tesseract"""
 
-    ocr_layout = ocr_agent.get_layout_from_image(
-        image=table_element_image,
-        ocr_languages=ocr_languages,
-    )
+    ocr_layout = ocr_agent.get_layout_from_image(image=table_element_image)
     table_tokens = []
-    for ocr_region in ocr_layout:
+    for i, text in enumerate(ocr_layout.texts):
         table_tokens.append(
             {
                 "bbox": [
-                    ocr_region.bbox.x1,
-                    ocr_region.bbox.y1,
-                    ocr_region.bbox.x2,
-                    ocr_region.bbox.y2,
+                    ocr_layout.x1[i],
+                    ocr_layout.y1[i],
+                    ocr_layout.x2[i],
+                    ocr_layout.y2[i],
                 ],
-                "text": ocr_region.text,
+                "text": text,
+                # 'table_tokens' is a list of tokens
+                # Need to be in a relative reading order
+                "span_num": i,
+                "line_num": 0,
+                "block_num": 0,
             }
         )
 
-    # 'table_tokens' is a list of tokens
-    # Need to be in a relative reading order
-    # If no order is provided, use current order
-    for idx, token in enumerate(table_tokens):
-        if "span_num" not in token:
-            token["span_num"] = idx
-        if "line_num" not in token:
-            token["line_num"] = 0
-        if "block_num" not in token:
-            token["block_num"] = 0
     return table_tokens
 
 
 def merge_out_layout_with_ocr_layout(
-    out_layout: List["LayoutElement"],
-    ocr_layout: List["TextRegion"],
+    out_layout: LayoutElements,
+    ocr_layout: TextRegions,
     supplement_with_ocr_elements: bool = True,
-) -> List["LayoutElement"]:
+    subregion_threshold: float = env_config.OCR_LAYOUT_SUBREGION_THRESHOLD,
+) -> LayoutElements:
     """
     Merge the out layout with the OCR-detected text regions on page level.
 
@@ -364,12 +361,21 @@ def merge_out_layout_with_ocr_layout(
     supplemented with the OCR layout.
     """
 
-    out_regions_without_text = [region for region in out_layout if not valid_text(region.text)]
+    if len(out_layout) == 0 or len(ocr_layout) == 0:
+        # what if od model finds nothing but ocr finds something? should we use ocr output at all
+        # currently we require some kind of bounding box, from `out_layout` to aggreaget ocr
+        # results. Can we just use ocr bounding boxes (gonna be many but at least we save
+        # information)
+        return out_layout
 
-    for out_region in out_regions_without_text:
-        out_region.text = aggregate_ocr_text_by_block(
-            ocr_layout,
-            out_region,
+    invalid_text_indices = [i for i, text in enumerate(out_layout.texts) if not valid_text(text)]
+    out_layout.texts = out_layout.texts.astype(object)
+
+    for idx in invalid_text_indices:
+        out_layout.texts[idx] = aggregate_embedded_text_by_block(
+            target_region=out_layout.slice([idx]),
+            source_regions=ocr_layout,
+            threshold=subregion_threshold,
         )
 
     final_layout = (
@@ -404,10 +410,10 @@ def aggregate_ocr_text_by_block(
 
 @requires_dependencies("unstructured_inference")
 def supplement_layout_with_ocr_elements(
-    layout: List["LayoutElement"],
-    ocr_layout: List["TextRegion"],
+    layout: LayoutElements,
+    ocr_layout: TextRegions,
     subregion_threshold: float = env_config.OCR_LAYOUT_SUBREGION_THRESHOLD,
-) -> List["LayoutElement"]:
+) -> LayoutElements:
     """
     Supplement the existing layout with additional OCR-derived elements.
 
@@ -417,10 +423,8 @@ def supplement_layout_with_ocr_elements(
     OCR-derived list. Then, it appends the remaining OCR-derived regions to the existing layout.
 
     Parameters:
-    - layout (List[LayoutElement]): A list of existing layout elements, each of which is
-                                    an instance of `LayoutElement`.
-    - ocr_layout (List[TextRegion]): A list of OCR-derived text regions, each of which is
-                                     an instance of `TextRegion`.
+    - layout (LayoutElements): A collection of existing layout elements in array structures
+    - ocr_layout (TextRegions): A collection of OCR-derived text regions in array structures
 
     Returns:
     - List[LayoutElement]: The final combined layout consisting of both the original layout
@@ -435,25 +439,32 @@ def supplement_layout_with_ocr_elements(
      threshold.
     """
 
+    from unstructured_inference.inference.layoutelement import LayoutElements
+
     from unstructured.partition.pdf_image.inference_utils import (
         build_layout_elements_from_ocr_regions,
     )
 
-    ocr_regions_to_remove: list[TextRegion] = []
-    for ocr_region in ocr_layout:
-        for el in layout:
-            ocr_region_is_subregion_of_out_el = ocr_region.bbox.is_almost_subregion_of(
-                el.bbox,
-                subregion_threshold,
+    if len(layout) == 0:
+        if len(ocr_layout) == 0:
+            return layout
+        else:
+            ocr_regions_to_add = ocr_layout
+    else:
+        mask = (
+            ~bboxes1_is_almost_subregion_of_bboxes2(
+                ocr_layout.element_coords, layout.element_coords, subregion_threshold
             )
-            if ocr_region_is_subregion_of_out_el:
-                ocr_regions_to_remove.append(ocr_region)
-                break
+            .sum(axis=1)
+            .astype(bool)
+        )
 
-    ocr_regions_to_add = [region for region in ocr_layout if region not in ocr_regions_to_remove]
-    if ocr_regions_to_add:
+        # add ocr regions that are not covered by layout
+        ocr_regions_to_add = ocr_layout.slice(mask)
+
+    if len(ocr_regions_to_add):
         ocr_elements_to_add = build_layout_elements_from_ocr_regions(ocr_regions_to_add)
-        final_layout = layout + ocr_elements_to_add
+        final_layout = LayoutElements.concatenate([layout, ocr_elements_to_add])
     else:
         final_layout = layout
 
