@@ -16,6 +16,7 @@ from unstructured.documents.elements import (
     Element,
     ElementMetadata,
 )
+from unstructured.file_utils.ndjson import dumps as ndjson_dumps
 from unstructured.partition.common.common import exactly_one
 from unstructured.utils import Point, dependency_exists, requires_dependencies
 
@@ -152,6 +153,29 @@ def elements_to_json(
     return json_str
 
 
+def elements_to_ndjson(
+    elements: Iterable[Element],
+    filename: Optional[str] = None,
+    encoding: str = "utf-8",
+) -> str:
+    """Serialize `elements` to a JSON array.
+
+    Also writes the JSON to `filename` if it is provided, encoded using `encoding`.
+
+    The JSON is returned as a string.
+    """
+    # -- serialize `elements` as a JSON array (str) --
+    precision_adjusted_elements = _fix_metadata_field_precision(elements)
+    element_dicts = elements_to_dicts(precision_adjusted_elements)
+    ndjson_str = ndjson_dumps(element_dicts, sort_keys=True)
+
+    if filename is not None:
+        with open(filename, "w", encoding=encoding) as f:
+            f.write(ndjson_str)
+
+    return ndjson_str
+
+
 def _fix_metadata_field_precision(elements: Iterable[Element]) -> list[Element]:
     out_elements: list[Element] = []
     for element in elements:
